/*----------------------------------------------------------------------------------*/
/*	Copyright (C) Endress+Hauser Metechnik GmbH+Co (2000). All Rights Reserved		*/
/*	Product:		SIMATIC PDM														*/
/*                                                          						*/
/*	Device:			ECHO LEVEL DEVICE for HART										*/
/*	Description:	Device Description. Part to import HCF-DD						*/
/*	Dictionary:		SIPADC01.dct													*/
/*                                                          						*/
/*  $Revision:1.4$	 							*/
/*  $Date:05.04.2002$								*/
/*  Obere Revision fr QVCS, untere fr DD, unten die aktuelle angeben 		*/ 
/*  Revision:1.4	 							*/
/*  Date:05.04.2002								*/
/*----------------------------------------------------------------------------------*/
/* Date:                25.06.2001													*/
/* Author:              J. Flegel / ICS GmbH										*/
/* Changed Function(s): Change001 Korrekturen bzgl. Konformittstest vom 18.06.01	*/
/*----------------------------------------------------------------------------------*/
/* Date:                27.06.2001													*/
/* Author:              J. Flegel / ICS GmbH										*/
/* Changed Function(s): Change002 Korrekturen bzgl. Konformittstest vom 18.06.01	*/
/*----------------------------------------------------------------------------------*/
/* Date:                29.06.2001													*/
/* Author:              J. Flegel / ICS GmbH										*/
/* Changed Function(s): Change003 MIN/MAX-Grenzen Behandlung						*/
/*----------------------------------------------------------------------------------*/
/* Date:                05.04.2002                                              */
/* Version:		1.4							*/
/* Author:              F.Birgel / PCPS     					*/
/* Changed Function(s): HART Standardansicht eingefhrt				*/
/*------------------------------------------------------------------------------*/
/* Date:                10.02.2009                                              */
/* Version:		1.5							*/
/* Author:              F.Birgel / PCPS     					*/
/* Changed Function(s): Bi change V1.5 fr PDM V6.0SP5				*/
/*------------------------------------------------------------------------------*/
/* defines from fmr23x.ddl */
#define LEVEL0		0
#define WRITABLE	1163

#define LIN_HEIGHT	240
#define LIN_ACTIVA	4
#define LIN_MANUAL	241
#define LIN_SEMI	242
#define LIN_CLEAR	243
#define LIN_LINEAR	0

#define SERVICE_LEVEL0	(service_index == LEVEL0)

#define SIM_OFF		0
#define SIM_LEVEL	1
#define SIM_VOLUME	2
#define SIM_CURRENT	3

#define DEF_MATRIX_ERRORS				\
	DEFAULT_VALUE 0;					\
	{0,   [E0]	,[E0_help]	},			\
	{101, [E101],[E101_help]},			\
	{102, [E102],[E102_help]},			\
	{103, [E103],[E103_help]},			\
	{106, [E106],[E106_help]},			\
	{110, [E110],[E110_help]},			\
	{111, [E111],[E111_help]},			\
	{112, [E112],[E112_help]},			\
	{113, [E113],[E113_help]},			\
	{114, [E114],[E114_help]},			\
	{115, [E115],[E115_help]},			\
	{116, [E116],[E116_help]},			\
	{121, [E121],[E121_help]},			\
	{231, [E231],[E231_help]},			\
	{232, [E232],[E232_help]},			\
	{512, [E512],[E512_help]},			\
	{602, [E602],[E602_help]},			\
	{604, [E604],[E604_help]},			\
	{605, [E605],[E605_help]},			\
	{613, [E613],[E613_help]},			\
	{620, [E620],[E620_help]},			\
	{621, [E621],[E621_help]},			\
	{641, [E641],[E641_help]},			\
	{643, [E643],[E643_help]}			

#define DEF_ACCESS_71_2_79(service_idx)										\
	IF ((security_locking == 333))											\
	{																		\
		IF (((service_locking == 333) || (service_locking == 1163))			\
			 && (service_index == service_idx))								\
		{																	\
			READ & WRITE;													\
		}																	\
		ELSE																\
		{																	\
			READ;															\
		}																	\
	}																		\
	ELSE																	\
	{																		\
		READ;																\
	}																		\


//Default Values:

//Change002 start
#define	FLOAT_MIN	-3.4E+38
#define	FLOAT_MAX	 3.4E+38
//Change002 end

#define DEF_LOCAL_VALUE_20_SIG(vorzeichen)	/*Bi change V1.5 fr PDM V6.0SP5*/						\
		SELECT (select_distance_unit)								\
		{															\
			CASE 45: 20.0 * vorzeichen;								\
			CASE 44: 20.0 * 3.280839895 * vorzeichen; /* 20.0 * 3.280839895 Change003 */	\
			DEFAULT: 20.0 * vorzeichen;								\
		}

#define DEF_LOCAL_VALUE_20											\
		DEF_LOCAL_VALUE_20_SIG(1.0)

#define DEF_LOCAL_VALUE_0_08										\
		SELECT (select_distance_unit)								\
		{															\
			CASE 45: 0.08;											\
			CASE 44: 0.26; /* 0.08 * 3.280839895 */				\
			DEFAULT: 0.08;											\
		}

#define DEF_LOCAL_VALUE_23_5										\
		SELECT (select_distance_unit)								\
		{															\
			CASE 45: 23.50;											\
			CASE 44: 77.10; /* 23.50 * 3.280839895 */				\
			DEFAULT: 23.50;											\
		}

#define DEF_LOCAL_VALUE_19999_SIG(vorzeichen)		/*Bi change V1.5 fr PDM V6.0SP5*/					\
		SELECT (select_distance_unit)								\
		{															\
			CASE 45: 19999.0 * vorzeichen;								\
			CASE 44: 65613.52 * vorzeichen; /* 19999.0 * 3.280839895 */	\
			DEFAULT: 19999.0 * vorzeichen;								\
		}

#define DEF_LOCAL_VALUE_19999											\
		DEF_LOCAL_VALUE_19999_SIG(1.0)



IMPORT ENDRESS_HAUSER _FMR23X, DEVICE_REVISION 2, DD_REVISION 1
{
	EVERYTHING;

	REDEFINITIONS
	{
		
		VARIABLE diagnostic_code
			{
				REDEFINE TYPE ENUMERATED(2)
				{
				 //DISPLAY_FORMAT "03u";
				  DEF_MATRIX_ERRORS
				}
			}
		
		VARIABLE last_diagnostic
			{
				REDEFINE TYPE ENUMERATED(2)
				{
				 //DISPLAY_FORMAT "03u";
				  DEF_MATRIX_ERRORS
				}
			}

		VARIABLE burst_command_number
			 {
			  REDEFINE TYPE ENUMERATED
		     {
			   DEFAULT_VALUE 1;
			   {1, [primary_variable] } ,
			   {2, [percent_range_and_current] } ,
			   {3, [all_process_variables_and_current] }
			  }
			 }

		VARIABLE burst_mode_select
			 {
			  REDEFINE TYPE	ENUMERATED
			  {
			   DEFAULT_VALUE 0;
			   {1, [on]},
			   {0, [off]}
			  }
			 }

		VARIABLE measured_parameter
		{
			REDEFINE TYPE		FLOAT
			{
				DISPLAY_FORMAT IF (linearization_mode == LIN_HEIGHT)
				{
				 "6.2f" ;
				}
				ELSE
				{
					SELECT (format)
					{
						CASE 0  :	"6.0f" ;
						CASE 1  :	"6.1f" ;
						CASE 2  :	"6.2f" ;
						CASE 3  :	"6.3f" ;
						CASE 4  : "6.4f" ;
						DEFAULT :	"6.0f" ;
					}
				}
				DELETE MIN_VALUE;
				DELETE MAX_VALUE;
			}
		}

		VARIABLE empty_value
		{
			REDEFINE TYPE		FLOAT
			{
				EDIT_FORMAT	"9.2f" ;
				DISPLAY_FORMAT	"9.2f" ;
				DEFAULT_VALUE	 DEF_LOCAL_VALUE_20;
				MIN_VALUE	 0.0;
				MAX_VALUE	 DEF_LOCAL_VALUE_20;
			}
		}

		VARIABLE full_value
		{
			REDEFINE TYPE		FLOAT
			{
				EDIT_FORMAT	"9.2f" ;
				DISPLAY_FORMAT	"9.2f" ;
				DEFAULT_VALUE	 DEF_LOCAL_VALUE_20;
				MIN_VALUE	 0.0;
				MAX_VALUE	 DEF_LOCAL_VALUE_20;
			}
		}

		VARIABLE delay_time
			{
			   REDEFINE LABEL	[label_delay_time];
			}
		VARIABLE quit_mode
			{
			   REDEFINE LABEL	[label_quit_mode];
			}
		VARIABLE plausibility_mode
			{
			   REDEFINE LABEL	[label_plausibility_mode];
			}
		VARIABLE echo_suppress
			{
			   REDEFINE LABEL	[label_echo_suppress];
			}

		VARIABLE distance_suppress
		{
			REDEFINE LABEL	[label_distance_suppress];
			TYPE		FLOAT
			{
				EDIT_FORMAT	"9.2f" ;
				DISPLAY_FORMAT "9.2f" ;
				DEFAULT_VALUE	DEF_LOCAL_VALUE_23_5;
				MIN_VALUE	0.00;
				MAX_VALUE	DEF_LOCAL_VALUE_23_5;
			}
		}

		VARIABLE echo_quality
		{
			REDEFINE LABEL	[label_echo_quality];
			ADD CONSTANT_UNIT	"dB";
		}

		VARIABLE micro_factor
			{
			   REDEFINE LABEL	[label_micro_factor];
			}
		VARIABLE first_echo
			{
			   REDEFINE LABEL	[label_first_echo];
			}

		VARIABLE window_suppress
		{
			REDEFINE LABEL	[label_window_suppress];
			TYPE		FLOAT
			{
				EDIT_FORMAT	"9.2f" ;
				DISPLAY_FORMAT	"9.2f" ;
				DEFAULT_VALUE	DEF_LOCAL_VALUE_0_08;
				MIN_VALUE		DEF_LOCAL_VALUE_0_08;
				MAX_VALUE		DEF_LOCAL_VALUE_20;
			}
		}

		VARIABLE offset
		{
			REDEFINE LABEL	[label_offset];
			TYPE		FLOAT
			{
				EDIT_FORMAT	"9.2f" ;
				DISPLAY_FORMAT	"9.2f" ;
				DEFAULT_VALUE	0.0;
				MIN_VALUE		DEF_LOCAL_VALUE_20_SIG(-1.0);
				MAX_VALUE		DEF_LOCAL_VALUE_20;
			}
		}

		VARIABLE sensor_lower_limit
			{
			   REDEFINE LABEL	[label_sensor_lower_limit];
			}

		VARIABLE sensor_upper_limit
			{
			   REDEFINE LABEL	[label_sensor_upper_limit];
			}
		VARIABLE select_distance_unit
		{
			REDEFINE LABEL	[label_select_distance_unit];
		}

		VARIABLE display_value_at_04_mA
			{
			   REDEFINE LABEL	[label_display_value_at_04_mA];
				TYPE		FLOAT
						{
						 DISPLAY_FORMAT IF (linearization_mode == LIN_HEIGHT)
								{
								 "8.2f" ;
								}
								ELSE
								{
								 SELECT (format)
				 				 {
								  CASE 0  :	"8.0f" ;
								  CASE 1  :	"8.1f" ;
								  CASE 2  :	"8.2f" ;
								  CASE 3  :	"8.3f" ;
								  CASE 4  : "8.4f" ;
								  DEFAULT :	"8.0f" ;
								 }
								}
						 EDIT_FORMAT	IF (linearization_mode == LIN_HEIGHT)
								{
								 "9.2f" ;
								}
								ELSE
								{
								 SELECT (format)
				 				 {
								  CASE 0  :	"8.0f" ;
								  CASE 1  :	"8.1f" ;
								  CASE 2  :	"8.2f" ;
								  CASE 3  :	"8.3f" ;
								  CASE 4  : "8.4f" ;
								  DEFAULT :	"8.0f" ;
								 }
								}
						DEFAULT_VALUE 0.0;
						 MIN_VALUE	-19999.0 ;
						 MAX_VALUE	 19999.0 ;
						}
			}

		VARIABLE display_value_at_20_mA
			{
			   REDEFINE LABEL	[label_display_value_at_20_mA];
				TYPE		FLOAT
						{
						 DISPLAY_FORMAT IF (linearization_mode == LIN_HEIGHT)
								{
								 "8.2f" ;
								}
								ELSE
								{
								 SELECT (format)
				 				 {
								  CASE 0  :	"8.0f" ;
								  CASE 1  :	"8.1f" ;
								  CASE 2  :	"8.2f" ;
								  CASE 3  :	"8.3f" ;
								  CASE 4  :	"8.4f" ;
								  DEFAULT :	"8.0f" ;
								 }
								}
						 EDIT_FORMAT	IF (linearization_mode == LIN_HEIGHT)
								{
								 "8.2f" ;
								}
								ELSE
								{
								 SELECT (format)
				 				 {
								  CASE 0  :	"8.0f" ;
								  CASE 1  :	"8.1f" ;
								  CASE 2  :	"8.2f" ;
								  CASE 3  :	"8.3f" ;
								  CASE 4  :     "8.4f" ;
								  DEFAULT :	"8.0f" ;
								 }
								}
						DEFAULT_VALUE 100.0;
						 MIN_VALUE	-19999.0 ;
						 MAX_VALUE	 19999.0 ;
						}
			}

		VARIABLE calibration_unit
			{
			   REDEFINE LABEL	[label_calibration_unit];
			}

		VARIABLE unit_after_lin
			{
			   REDEFINE LABEL	[label_unit_after_lin];
			}

		VARIABLE alarm_selection_code_1
			{
			   REDEFINE LABEL	[label_alarm_selection_code_1];
			}

		VARIABLE linearization_mode
			{
			   REDEFINE LABEL	[label_linearization_mode];
			}

		VARIABLE level_damping
			{
			   REDEFINE LABEL	[label_level_damping];
			}
		VARIABLE current_output_mode
			{
			   REDEFINE LABEL	[label_current_output_mode];
			}
		VARIABLE application
			{
			   REDEFINE LABEL	[label_application];
			}
		
		VARIABLE instrument_address
			{
			   REDEFINE LABEL	[label_instrument_address];
			}

		VARIABLE instr_and_softw_no
			{
			   REDEFINE LABEL	[label_instr_and_softw_no];
			}

		VARIABLE tag
			{
			   REDEFINE LABEL	"Set tag number|de|Messtelle";
			}
		VARIABLE device_status
			{
				REDEFINE LABEL	[device_status] ;
				REDEFINE STYLE		"MULTILINE 30 7";
			}

		VARIABLE simulation_mode
		{
			REDEFINE CLASS	SERVICE & DYNAMIC; //Change002
			REDEFINE POST_EDIT_ACTIONS
			{
				post_simulation_mode
			}
			REDEFINE POST_READ_ACTIONS
			{
				post_simulation_mode
			}
		}

		VARIABLE select_distance_unit
		{
			REDEFINE POST_EDIT_ACTIONS
			{
				post_select_distance_unit
			}
		}

		VARIABLE xmtr_specific_status_0
		{
			REDEFINE STYLE		"MULTILINE 30 7";
		}

		VARIABLE xmtr_specific_status_1
		{
			REDEFINE STYLE		"MULTILINE 30 7";
		}

		VARIABLE xmtr_specific_status_2
		{
			REDEFINE STYLE		"MULTILINE 30 7";
		}

		VARIABLE xmtr_specific_status_3
		{
			REDEFINE STYLE		"MULTILINE 30 7";
		}

		VARIABLE xmtr_specific_status_4
		{
			REDEFINE STYLE		"MULTILINE 30 7";
		}

		VARIABLE xmtr_specific_status_5
		{
			REDEFINE STYLE		"MULTILINE 30 7";
		}


		VARIABLE xy_table_index_1
		{
			REDEFINE LABEL	[label_xy_table_index_1];
			DELETE POST_EDIT_ACTIONS;
		}

		VARIABLE xy_table_X
		{
			REDEFINE LABEL	[label_xy_table_X];
			TYPE	FLOAT
			{
				DISPLAY_FORMAT "9.3f" ;
				EDIT_FORMAT    "9.3f" ;
				MIN_VALUE		0.0;
				MAX_VALUE		DEF_LOCAL_VALUE_20;
			}
			DELETE	POST_EDIT_ACTIONS;
		}


		VARIABLE xy_table_Y
		{
			REDEFINE LABEL	[label_xy_table_Y];
			DELETE POST_EDIT_ACTIONS;
		}

		VARIABLE vmax
		{
			REDEFINE LABEL	[label_vmax];
			DELETE POST_EDIT_ACTIONS;
		}

		VARIABLE service_index
		{
			REDEFINE LABEL	[label_service_index];
			DELETE POST_EDIT_ACTIONS;
		}

		VARIABLE z_distance	//V7H1
		{
			REDEFINE LABEL	[label_z_distance];
			REDEFINE TYPE		FLOAT
			{
				DISPLAY_FORMAT	"10.3f" ;
				EDIT_FORMAT	"10.3f" ;
				DEFAULT_VALUE	0.0;
				//Change002 start
				//MIN_VALUE		IF (service_index == 0)	{0.0;} ELSE {-19999.0;}
				//MAX_VALUE		IF (service_index == 0) {DEF_LOCAL_VALUE_20;} ELSE {19999.0;}
				MIN_VALUE		FLOAT_MIN;
				MAX_VALUE		FLOAT_MAX;
				//Change002 end
			}
			DELETE POST_EDIT_ACTIONS;
			REDEFINE HANDLING DEF_ACCESS_71_2_79(service_index)
		}

		VARIABLE type_of_antenna //V7H2
		{
			REDEFINE LABEL	[label_type_of_antenna];
			REDEFINE TYPE		FLOAT
			{
				DISPLAY_FORMAT	"10.3f" ;
				EDIT_FORMAT	"10.3f" ;
				DEFAULT_VALUE	0.0;
				//Change002 start
				//MIN_VALUE		IF (service_index == 0)	{DEF_LOCAL_VALUE_20_SIG(-1.0);} ELSE {-19999.0;}
				//MAX_VALUE		IF (service_index == 0)	{DEF_LOCAL_VALUE_20;} ELSE {19999.0;}
				MIN_VALUE		FLOAT_MIN;
				MAX_VALUE		FLOAT_MAX;
				//Change002 end
			}
			DELETE POST_EDIT_ACTIONS;
			REDEFINE HANDLING DEF_ACCESS_71_2_79(service_index)
		}

		VARIABLE module_type	//V7H3
		{
			REDEFINE LABEL	[label_module_type];
			DELETE POST_EDIT_ACTIONS;
			REDEFINE HANDLING DEF_ACCESS_71_2_79(service_index)
			DELETE VALIDITY;
		}

		VARIABLE record_tvg		//V7H4
		{
			REDEFINE LABEL	[label_record_tvg];
			DELETE POST_EDIT_ACTIONS;
			REDEFINE HANDLING DEF_ACCESS_71_2_79(service_index)
			DELETE VALIDITY;
		}

		VARIABLE tank_status	//V7H5
		{
			REDEFINE LABEL	[label_tank_status];
			DELETE POST_EDIT_ACTIONS;
			REDEFINE HANDLING DEF_ACCESS_71_2_79(1)
//Change002 start
   			REDEFINE TYPE		FLOAT
					{
					 DISPLAY_FORMAT	IF (SERVICE_LEVEL0)
							{
							 "9.0f" ;
							}
							ELSE			/* Mapping for service_1 (S1-V7H5) */
							{
							 "9.2f" ;
							}
					  EDIT_FORMAT  	IF (SERVICE_LEVEL0)
							{
							 "9.0f" ;
							}
							ELSE			/* Mapping for service_1 (S1-V7H5) */
							{
							 "9.2f" ;
							}
					 MIN_VALUE		FLOAT_MIN;
					 MAX_VALUE		FLOAT_MAX;
					}
//Change002 end
		}

		VARIABLE minimum_sn		//V7H6
		{
			REDEFINE LABEL	[label_minimum_sn];
			DELETE POST_EDIT_ACTIONS;
			REDEFINE HANDLING DEF_ACCESS_71_2_79(0)
			ADD CONSTANT_UNIT	IF (service_index == 0) {"dB";}
//Change002 start
			REDEFINE TYPE	FLOAT
			{
			 DISPLAY_FORMAT	IF (SERVICE_LEVEL0)	/* Mapping for service_1 (S1-V7H6) */
					{
					 "3.0f" ;
					}
					ELSE
					{
					 "6.0f" ;
					}
			  EDIT_FORMAT	"3.0f" ;
			  MIN_VALUE		FLOAT_MIN;
			  MAX_VALUE		FLOAT_MAX;
			}
//Change002 end
		}

		VARIABLE mam_configuration	//V7H7
		{
			REDEFINE LABEL	[label_mam_configuration];
			DELETE POST_EDIT_ACTIONS;
			REDEFINE HANDLING DEF_ACCESS_71_2_79(0)
//Change002 start
    		REDEFINE TYPE		FLOAT
			{
			 DISPLAY_FORMAT	IF (SERVICE_LEVEL0)
					{
					 "4.0f" ;
					}
					ELSE			/* Mapping for service_1 (S1-V7H7) */
					{
					 "6.0f" ;
					}
			 EDIT_FORMAT    "4.0f" ;
			 MIN_VALUE		FLOAT_MIN;
			 MAX_VALUE		FLOAT_MAX;
		   	}
//Change002 end

		}

		VARIABLE uncorrected_distance	//V7H8
		{
			REDEFINE LABEL	[label_uncorrected_distance];
			DELETE POST_EDIT_ACTIONS;
			DELETE VALIDITY;
//Change002 start
    		REDEFINE TYPE		FLOAT
			{
				 DISPLAY_FORMAT "5.2f" ;
				 MIN_VALUE		IF (select_distance_unit >= 0) {FLOAT_MIN;}  //Change003: diese damit PDM nicht mehr auf -0.0 umrechnet
				 MAX_VALUE		IF (select_distance_unit >= 0) {FLOAT_MAX;}  //Change003: sol fr alle Flle gelten, daher Abfrage >= 0
			}
//Change002 end
		}

		VARIABLE service_locking	//V7H9
		{
			DELETE POST_EDIT_ACTIONS;
		}

		VARIABLE default_values
		{
			REDEFINE TYPE		ENUMERATED (2)
			{
				DEFAULT_VALUE 333;
				{ 0,		[no_function]			},
				{ 333,		[customer_reset]		},	//Kundenreset
				{ 7864, 	[full_reset]			},	//Totalreset
				{ 545, 		[tvg_reset] 			}	//TVG Reset
			}
		}

		VARIABLE simulation_value
		{
			REDEFINE TYPE		FLOAT
					{
					 EDIT_FORMAT	SELECT (simulation_mode)
         			     			{
							CASE SIM_CURRENT: "5.2f" ;
							CASE SIM_LEVEL  : "5.2f" ;
            				DEFAULT	 : "8.3f" ;
        		 					}
					 DISPLAY_FORMAT SELECT (simulation_mode)
         						{
							 CASE SIM_CURRENT: "5.2f" ;
							 CASE SIM_LEVEL  : "5.2f" ;
							 DEFAULT       	 : "8.3f" ;
							}
					 MIN_VALUE	SELECT (simulation_mode)
							{
							 CASE SIM_LEVEL  : 0.0 ;
							 CASE SIM_VOLUME : -19999.0 ;
							 CASE SIM_CURRENT: 3.8 ;
							 DEFAULT         : -19999.0 ;
							}
					 MAX_VALUE	SELECT (simulation_mode)
							{
								 CASE SIM_LEVEL	 : 20.0 ;
								 CASE SIM_VOLUME : 19999.0 ;
								 CASE SIM_CURRENT: 22.0 ;
								 DEFAULT         : 19999.0 ;
								 }
					 DEFAULT_VALUE	SELECT (simulation_mode)
							{
							 CASE SIM_LEVEL  : 0.0 ;
							 CASE SIM_VOLUME : 0.0 ;
							 CASE SIM_CURRENT: 3.8 ;
							 DEFAULT         : 0.0 ;
							}
					}
			VALIDITY	IF (simulation_mode == SIM_OFF)
					{
					 FALSE ;
					}
					ELSE
					{
					 TRUE ;
					}
		}


		// Beim Auswerten der UNIT-Relation von select_distance_unit
		// sucht PDM nach dem Kommando von select_distance_unit
		// Diese Unit muss ber den Index 0x82 gelesen werden. PDM
		// findet aber schon ein Vorkommen bei Index 0x01. 
        ARRAY OF VARIABLE variable_array_unit
        {
           REDEFINE ELEMENTS	{
              0x00, calibration_unit ;

              0x01, nul_unit_a;//select_distance_unit ;
              0x02, nul_unit_a;//select_distance_unit ;
              0x03, nul_unit_a ;
              0x04, time_unit ;

              0x05, calibration_unit ;
              0x06, calibration_unit ;
              0x07, nul_unit_a;
              0x08, nul_unit_a;//select_distance_unit ;
              0x09, nul_unit_a;//select_distance_unit ;

              0x20, nul_unit_a;
              0x21, nul_unit_a ;
              0x22, nul_unit_a;//select_distance_unit ;
              0x23, displayed_unit ;
              0x25, displayed_unit ;

              0x30, nul_unit_a ;
              0x31, nul_unit_a;//select_distance_unit ;
              0x32, nul_unit_a ;
              0x33, nul_unit_a ;
              0x34, nul_unit_a ;
              0x35, nul_unit_a;//select_distance_unit ;
              0x36, nul_unit_a;//select_distance_unit ;

              0x70, nul_unit_a ;
              0x71, nul_unit_a;//select_distance_unit ;
              0x72, nul_unit_a;//select_distance_unit ;

              0x73, nul_unit_a ;

              0x74, nul_unit_b ;
              0x75, nul_unit_b ;
              0x76, nul_unit_b ;
              0x77, nul_unit_b ;
              0x78, nul_unit_a;//select_distance_unit ;
              0x79, nul_unit_b ;

              0x81, nul_unit_b ;
              0x82, nul_unit_b ;
              0x83, time_unit ;
              0x85, nul_unit_b ;
              0x86, nul_unit_b ;

              0x90, nul_unit_b ;
              0x91, nul_unit_b ;
              0x93, nul_unit_b ;
              0x94, nul_unit_b ;
              0x95, nul_unit_b ;
              0x96, nul_unit_b ;
              0x97, simulation_unit ;
              0x98, current_unit ;
              0x99, nul_unit_b ;

              0xA0, nul_unit_b ;
              0xA3, nul_unit_b ;
              }
        }

		REDEFINE UNIT select_distance_unit_relation
		{
			select_distance_unit :					/* V8H2 */
						empty_value ,				/* V0H1 */
			  			full_value ,				/* V0H2 */
						fill_distance ,				/* V0H8 */
			  			fill_height ,				/* V0H9 */
						xy_table_X ,				/* V2H2 */
						distance_suppress ,			/* V3H1 */
						window_suppress				/* V3H5 */
						offset ,					/* V3H6 */
						uncorrected_distance,		/* V7H8 */
//Change002 start
#ifdef MICRO_COND_IN_UNIT
						IF (service_index == 0)
						{
							z_distance ,			/* V7H1 */
							type_of_antenna			/* V7H2 */
						}
#endif
//Change002 end
		}

		REDEFINE UNIT calibration_unit_relation
		{
			calibration_unit:					/* HART */
						measured_parameter ,		/* V0H0 */
						display_value_at_04_mA ,	/* V0H5 */
						display_value_at_20_mA ,	/* V0H6 */
						//xy_table_Y ,			/* V2H3 */ // siehe: unit_after_lin_relation
						//vmax 					/* V2H5 */ // siehe: unit_after_lin_relation

		}

		MENU COMMUNICATION
		{
			REDEFINE LABEL	[label_COMMUNICATION];
			REDEFINE ITEMS	
			{
				matrix_par7.tag ,				/* HART */
				matrix_par7.unit_after_lin		/* VAH3 */
			}
		}

		MENU SIMULATION
		{
			REDEFINE LABEL	[label_SIMULATION];
			REDEFINE ITEMS	
			{
				matrix_par6.diagnostic_code ,			
				matrix_par6.last_diagnostic ,			
				matrix_par6.instr_and_softw_no ,		
				help_instrument_address ,		
				help_default_values,      		
				help_simulation_mode ,			
				matrix_par6.simulation_value ,			
				matrix_par6.output_current ,			
				help_security_locking 			
			}
		}
		
		MENU SERVICES
		{
			REDEFINE ACCESS OFFLINE; //Change002
			REDEFINE LABEL [label_SERVICE];
			REDEFINE ITEMS	
			{
				help_V7,
				matrix_par4.service_index ,			
				matrix_par4.z_distance			(READ_ONLY),			
				matrix_par4.type_of_antenna 	(READ_ONLY),		
				matrix_par4.module_type 		(READ_ONLY),			
				matrix_par4.record_tvg 			(READ_ONLY),			
				matrix_par4.tank_status 		(READ_ONLY),			
				matrix_par4.minimum_sn 			(READ_ONLY),			
				matrix_par4.mam_configuration 	(READ_ONLY),		
				matrix_par4.uncorrected_distance(READ_ONLY),	
				matrix_par4.service_locking 	(READ_ONLY)	
			}
		}

		MENU OPERATOR
		{
			REDEFINE LABEL [label_BETR_PARAMTER];
			REDEFINE ITEMS
			{
				matrix_par5.current_output_mode ,		
				matrix_par5.select_distance_unit ,		
				matrix_par5.delay_time ,				
				matrix_par5.quit_mode					
				matrix_par5.plausibility_mode			
			}
		}


		MENU EXTENDED_CALIBRATION
		{
			REDEFINE LABEL [label_EXTENDED_CALIBRATION];
			REDEFINE ITEMS	
			{
				help_echo_suppress,					
				matrix_par3.distance_suppress	(READ_ONLY),				
				matrix_par3.echo_quality ,			        
				matrix_par3.micro_factor ,					
				matrix_par3.first_echo ,					
				matrix_par3.window_suppress ,				
				matrix_par3.offset							
			}
		}
		
		MENU LINEARIZATION
		{
			REDEFINE LABEL [label_LINEARIZATION] ;
			REDEFINE ITEMS	
			{
				help_linearization_mode ,	//Change002	
				help_xy_table_index_1 ,          
				help_xy_table_X ,                
				help_xy_table_Y ,                
				help_vmax                        
			}
		}

		MENU CALIBRATION
		{
		   REDEFINE LABEL [label_CALIBRATION];
		   REDEFINE ITEMS
		   {
				matrix_par1.measured_parameter,                            
				matrix_par1.empty_value,                                   
				matrix_par1.full_value,                                    
				help_application,//matrix_par1.application,                                   
				matrix_par1.level_damping,                                 
				matrix_par1.display_value_at_04_mA,                        
				matrix_par1.display_value_at_20_mA,                        
				matrix_par1.alarm_selection_code_1,                        
				matrix_par1.fill_distance,                                 
				matrix_par1.fill_height                                    
		   }
		}

		REDEFINE MENU upload_variables
		{
			LABEL		[menu_load_to_device];
			PRE_WRITE_ACTIONS
			{
				pre_write_upload_variables
			}
			ITEMS
			{

				IF (local_abort == 0)
				{
					tag,
					descriptor,
					message,
					date,
					final_assembly_number,
					select_distance_unit,		//Change002
					unit_after_lin,				//Change002
					empty_value,				/* V0H1 */
					full_value,					/* V0H2 */

					level_damping,
					//application,				//Dialog
					display_value_at_04_mA,
					display_value_at_20_mA,
					alarm_selection_code_1,
					//linearization_mode,		//Dialog Change002
					//xy_table_index_1,			//Dialog
					//xy_table_X,				//Dialog
					//xy_table_Y,				//Dialog
					//vmax,						//Dialog

					//echo_suppress,			//Dialog
					//distance_suppress,		//Dialog
					micro_factor,
					first_echo,
					window_suppress,
					offset,

					// alle im Dialog schreibbar:
					service_index,
					//z_distance,
					//type_of_antenna,
					//module_type,
					//record_tvg,
					//minimum_sn,
					//mam_configuration,
					//service_locking,

					current_output_mode,
					delay_time,
					//instrument_address,	// Dialog: Adresse vergeben
					quit_mode,
					plausibility_mode,
				} 
			}
		}

	}
	
}

/************************************************************************************/

UNIT unit_after_lin_relation
{
	displayed_unit/*unit_after_lin*/:					/* VAH3 */
				xy_table_Y ,		/* V2H3 */
				vmax 				/* V2H5 */

}

METHOD post_simulation_mode
{
	LABEL		"post_simulation_mode";
	DEFINITION
	{
		switch (simulation_mode)
		{
		case 1: simulation_unit = select_distance_unit;
				break;
		case 2: simulation_unit = calibration_unit;
				break;
		case 3: simulation_unit = 39;
				break;
		}
	}
}


METHOD post_select_distance_unit
{
	LABEL		"post_select_distance_unit";
	DEFINITION
	{
		displayed_unit = select_distance_unit;
	}
}

METHOD pre_write_upload_variables
{
	CLASS		INPUT;
	DEFINITION
	{
		int ret, i;
		local_abort = 0; //kein Abbruch

	}
}

COLLECTION OF VARIABLE matrix_par1
{

    MEMBERS
    {
		measured_parameter, measured_parameter,				[V0H0_LABEL_change];
		empty_value, empty_value,							[V0H1_LABEL_change];
		full_value, full_value,								[V0H2_LABEL_change];
		application, application,							[V0H3_LABEL_change];
		level_damping, level_damping,						[V0H4_LABEL_change];
		display_value_at_04_mA, display_value_at_04_mA,		[V0H5_LABEL_change];
		display_value_at_20_mA, display_value_at_20_mA,		[V0H6_LABEL_change];
		alarm_selection_code_1, alarm_selection_code_1,		[V0H7_LABEL_change];
		fill_distance, fill_distance,						[V0H8_LABEL_change];
		fill_height, fill_height,							[V0H9_LABEL_change];

    }
}



COLLECTION OF VARIABLE matrix_par2
{

    MEMBERS
    {
		linearization_mode, linearization_mode,				[V2H0_LABEL_change];
		xy_table_index_1, xy_table_index_1,					[V2H1_LABEL_change];
		xy_table_X, xy_table_X,								[V2H2_LABEL_change];
		xy_table_Y, xy_table_Y,								[V2H3_LABEL_change];
		vmax, vmax,											[V2H5_LABEL_change];
    }
}


COLLECTION OF VARIABLE matrix_par3
{

    MEMBERS
    {															
		echo_suppress, echo_suppress,						[V3H0_LABEL_change];
		distance_suppress, distance_suppress,				[V3H1_LABEL_change];
		echo_quality, echo_quality,							[V3H2_LABEL_change];
		micro_factor, micro_factor,							[V3H3_LABEL_change];
		first_echo, first_echo,								[V3H4_LABEL_change];
		window_suppress, window_suppress,					[V3H5_LABEL_change];
		offset, offset,										[V3H6_LABEL_change];
    }															
}

COLLECTION OF VARIABLE matrix_par4
{

    MEMBERS
    {
		service_index, service_index,						[V7H0_LABEL_change];
		IF (service_index == 0)
		{
			z_distance, z_distance,								[V7H1_LABEL_change];
			type_of_antenna, type_of_antenna,					[V7H2_LABEL_change];
		}
		ELSE
		{
			z_distance, z_distance,								[V7H1_LABEL_change_1];
			type_of_antenna, type_of_antenna,					[V7H2_LABEL_change_1];
		}

		module_type, module_type,							[V7H3_LABEL_change];
		record_tvg, record_tvg,								[V7H4_LABEL_change];
		tank_status, tank_status,							[V7H5_LABEL_change];
		minimum_sn, minimum_sn,								[V7H6_LABEL_change];
		mam_configuration, mam_configuration,				[V7H7_LABEL_change];
		uncorrected_distance, uncorrected_distance,			[V7H8_LABEL_change];
		service_locking, service_locking,					[V7H9_LABEL_change];
	}
}

COLLECTION OF VARIABLE matrix_par5
{

    MEMBERS
    {
		current_output_mode, current_output_mode,			[V8H1_LABEL_change];
		select_distance_unit, select_distance_unit,			[V8H2_LABEL_change];
		delay_time, delay_time,								[V8H3_LABEL_change];
		quit_mode, quit_mode,								[V8H5_LABEL_change];
		plausibility_mode, plausibility_mode,				[V8H6_LABEL_change];

	}
}


COLLECTION OF VARIABLE matrix_par6
{

    MEMBERS
    {
		diagnostic_code, diagnostic_code,					[V9H0_LABEL_change],[V9H0_LABEL_change_help];
		last_diagnostic, last_diagnostic,					[V9H1_LABEL_change],[V9H0_LABEL_change_help];
		instr_and_softw_no, instr_and_softw_no,				[V9H3_LABEL_change];
		instrument_address, instrument_address,				[V9H4_LABEL_change];
		default_values, default_values,						[V9H5_LABEL_change];
		simulation_mode, simulation_mode,					[V9H6_LABEL_change];
		simulation_value, simulation_value,					[V9H7_LABEL_change];
		output_current, output_current,						[V9H8_LABEL_change];
		security_locking, security_locking,					[V9H9_LABEL_change];
	}														
}

COLLECTION OF VARIABLE matrix_par7
{

    MEMBERS
    {
		tag, tag,											[VAH0_LABEL_change];
		unit_after_lin, unit_after_lin,						[VAH3_LABEL_change];
	}														
}

VARIABLE help_echo_suppress
{
	LABEL		[V3H0_LABEL_change];		// 
	HELP		[V3H0_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_echo_suppress]	},

	}
	HANDLING READ;
}

VARIABLE help_distance_suppress
{
	LABEL		[V3H1_LABEL_change];		// 
	HELP		[V3H1_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_distance_suppress]	},

	}
	HANDLING READ;
}


VARIABLE help_instrument_address
{
	LABEL		[V9H4_LABEL_change];		// 
	HELP		[V9H4_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_instrument_address]	},

	}
	HANDLING READ;
}

VARIABLE help_default_values
{
	LABEL		[V9H5_LABEL_change];		// 
	HELP		[V9H5_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_default_values]	},

	}
	HANDLING READ;
}

VARIABLE help_simulation_mode
{
	LABEL		[V9H6_LABEL_change];		// 
	HELP		[V9H6_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_simulation_mode]	},

	}
	HANDLING READ;
}

VARIABLE help_xy_table_index_1
{
	LABEL		[V2H1_LABEL_change];		// 
	HELP		[V2H1_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_tabele]	},

	}
	HANDLING READ;
}



VARIABLE help_xy_table_X
{
	LABEL		[V2H2_LABEL_change];		// 
	HELP		[V2H2_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_tabele]	},

	}
	HANDLING READ;
}

VARIABLE help_xy_table_Y
{
	LABEL		[V2H3_LABEL_change];		// 
	HELP		[V2H3_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_tabele]	},

	}
	HANDLING READ;
}

VARIABLE help_vmax
{
	LABEL		[V2H5_LABEL_change];		// 
	HELP		[V2H5_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_tabele]	},

	}
	HANDLING READ;
}

VARIABLE help_security_locking
{
	LABEL		[V9H9_LABEL_change];		// 
	HELP		[V9H9_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_security_locking]	},

	}
	HANDLING READ;
}

VARIABLE help_service_index
{
	LABEL		[V7H0_LABEL_change];		// 
	HELP		[V7H0_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_service_index]	},

	}
	HANDLING READ;
}

VARIABLE help_service_locking
{
	LABEL		[V7H9_LABEL_change];		// 
	HELP		[V7H9_LABEL_change_help];	// 
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_service_index]	},

	}
	HANDLING READ;
}

VARIABLE help_V7
{
	LABEL       [help_V7];
	CLASS       LOCAL;
	TYPE        ENUMERATED
	{
		DEFAULT_VALUE 0;
		{   0,  [enum_service_index]}
	}
	HANDLING    READ;
}

VARIABLE help_application
{
	LABEL		[V0H3_LABEL_change];
	HELP		[V0H3_LABEL_change_help];
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;

		{ 0,	[enum_empty_full_cal]	},

	}
	HANDLING READ;
}

//Change002 start
VARIABLE help_linearization_mode
{
	LABEL		[V2H0_LABEL_change];
	HELP		[V2H0_LABEL_change_help];
	CLASS		LOCAL;
	TYPE		ENUMERATED (1)
	{
		DEFAULT_VALUE	0;
		{ 0,	[enum_tabele]	},

	}
	HANDLING READ;
}
//Change002 end
